==> pub_sub-1.0.0.tar.gz: lib/src/json_rpc/server.dart <==
import 'dart:async';
import 'package:stream_channel/stream_channel.dart';
import 'package:json_rpc_2/json_rpc_2.dart' as json_rpc_2;
import 'package:uuid/uuid.dart';
import '../../pub_sub.dart';

/// A [Adapter] implementation that communicates via JSON RPC 2.0.
class JsonRpc2Adapter extends Adapter {
  final StreamController<PublishRequest> _onPublish =
      new StreamController<PublishRequest>();
  final StreamController<SubscriptionRequest> _onSubscribe =
      new StreamController<SubscriptionRequest>();
  final StreamController<UnsubscriptionRequest> _onUnsubscribe =
      new StreamController<UnsubscriptionRequest>();

  final List<json_rpc_2.Peer> _peers = [];
  final Uuid _uuid = new Uuid();

  json_rpc_2.Peer _peer;

  /// A [Stream] of incoming clients, who can both send and receive string data.
  final Stream<StreamChannel<String>> clientStream;

  JsonRpc2Adapter(this.clientStream);

  @override
  Stream<PublishRequest> get onPublish => _onPublish.stream;

  @override
  Stream<SubscriptionRequest> get onSubscribe => _onSubscribe.stream;

  @override
  Stream<UnsubscriptionRequest> get onUnsubscribe => _onUnsubscribe.stream;

  @override
  Future close() {
    if (_peer?.isClosed != true) _peer?.close();

    Future
        .wait(_peers.where((s) => !s.isClosed).map((s) => s.close()))
        .then((_) => _peers.clear());
    return new Future.value();
  }

  @override
  void start() {
    clientStream.listen((client) {
      var peer = _peer = new json_rpc_2.Peer(client);

      peer.registerMethod('publish', (json_rpc_2.Parameters params) async {
        var requestId = params['request_id'].asString;
        var clientId = params['client_id'].asString;
        var eventName = params['event_name'].asString;
        var value = params['value'].value;
        var rq = new _JsonRpc2PublishRequestImpl(
            requestId, clientId, eventName, value, peer);
        _onPublish.add(rq);
      });

      peer.registerMethod('subscribe', (json_rpc_2.Parameters params) async {
        var requestId = params['request_id'].asString;
        var clientId = params['client_id'].asString;
        var eventName = params['event_name'].asString;
        var rq = new _JsonRpc2SubscriptionRequestImpl(
            clientId, eventName, requestId, peer, _uuid);
        _onSubscribe.add(rq);
      });

      peer.registerMethod('unsubscribe', (json_rpc_2.Parameters params) async {
        var requestId = params['request_id'].asString;
        var clientId = params['client_id'].asString;
        var subscriptionId = params['subscription_id'].asString;
        var rq = new _JsonRpc2UnsubscriptionRequestImpl(
            clientId, subscriptionId, peer, requestId);
        _onUnsubscribe.add(rq);
      });

      peer.listen();
    });
  }
}

class _JsonRpc2PublishRequestImpl extends PublishRequest {
  final String requestId, clientId, eventName;
  final value;
  final json_rpc_2.Peer peer;

  _JsonRpc2PublishRequestImpl(
      this.requestId, this.clientId, this.eventName, this.value, this.peer);

  @override
  void accept(PublishResponse response) {
    peer.sendNotification(requestId, {
      'status': true,
      'request_id': requestId,
      'result': {'listeners': response.listeners}
    });
  }

  @override
  void reject(String errorMessage) {
    peer.sendNotification(requestId, {
      'status': false,
      'request_id': requestId,
      'error_message': errorMessage
    });
  }
}

class _JsonRpc2SubscriptionRequestImpl extends SubscriptionRequest {
  @override
  final String clientId, eventName;

  final String requestId;

  final json_rpc_2.Peer peer;

  final Uuid _uuid;

  _JsonRpc2SubscriptionRequestImpl(
      this.clientId, this.eventName, this.requestId, this.peer, this._uuid);

  @override
  FutureOr<Subscription> accept() {
    var id = _uuid.v4();
    peer.sendNotification(requestId,
        {'status': true, 'request_id': requestId, 'subscription_id': id});
    return new _JsonRpc2SubscriptionImpl(clientId, id, eventName, peer);
  }

  @override
  void reject(String errorMessage) {
    peer.sendNotification(requestId, {
      'status': false,
      'request_id': requestId,
      'error_message': errorMessage
    });
  }
}

class _JsonRpc2SubscriptionImpl extends Subscription {
  @override
  final String clientId, id;

  final String eventName;

  final json_rpc_2.Peer peer;

  _JsonRpc2SubscriptionImpl(this.clientId, this.id, this.eventName, this.peer);

  @override
  void dispatch(event) {
    peer.sendNotification('event', {'event_name': eventName, 'value': event});
  }
}

class _JsonRpc2UnsubscriptionRequestImpl extends UnsubscriptionRequest {
  @override
  final String clientId;

  @override
  final String subscriptionId;

  final json_rpc_2.Peer peer;

  final String requestId;

  _JsonRpc2UnsubscriptionRequestImpl(
      this.clientId, this.subscriptionId, this.peer, this.requestId);

  @override
  accept() {
    peer.sendNotification(requestId, {'status': true, 'result': {}});
  }

  @override
  void reject(String errorMessage) {
    peer.sendNotification(requestId, {
      'status': false,
      'request_id': requestId,
      'error_message': errorMessage
    });
  }
}

==> pub_sub-2.0.0.tar.gz: lib/src/json_rpc/server.dart <==
import 'dart:async';
import 'package:stream_channel/stream_channel.dart';
import 'package:json_rpc_2/json_rpc_2.dart' as json_rpc_2;
import 'package:uuid/uuid.dart';
import '../../pub_sub.dart';

/// A [Adapter] implementation that communicates via JSON RPC 2.0.
class JsonRpc2Adapter extends Adapter {
  final StreamController<PublishRequest> _onPublish =
      new StreamController<PublishRequest>();
  final StreamController<SubscriptionRequest> _onSubscribe =
      new StreamController<SubscriptionRequest>();
  final StreamController<UnsubscriptionRequest> _onUnsubscribe =
      new StreamController<UnsubscriptionRequest>();

  final List<json_rpc_2.Peer> _peers = [];
  final Uuid _uuid = new Uuid();

  json_rpc_2.Peer _peer;

  /// A [Stream] of incoming clients, who can both send and receive string data.
  final Stream<StreamChannel<String>> clientStream;

  JsonRpc2Adapter(this.clientStream);

  @override
  Stream<PublishRequest> get onPublish => _onPublish.stream;

  @override
  Stream<SubscriptionRequest> get onSubscribe => _onSubscribe.stream;

  @override
  Stream<UnsubscriptionRequest> get onUnsubscribe => _onUnsubscribe.stream;

  @override
  Future close() {
    if (_peer?.isClosed != true) _peer?.close();

    Future
        .wait(_peers.where((s) => !s.isClosed).map((s) => s.close()))
        .then((_) => _peers.clear());
    return new Future.value();
  }

  @override
  void start() {
    clientStream.listen((client) {
      var peer = _peer = new json_rpc_2.Peer(client);

      peer.registerMethod('publish', (json_rpc_2.Parameters params) async {
        var requestId = params['request_id'].asString;
        var clientId = params['client_id'].asString;
        var eventName = params['event_name'].asString;
        var value = params['value'].value;
        var rq = new _JsonRpc2PublishRequestImpl(
            requestId, clientId, eventName, value, peer);
        _onPublish.add(rq);
      });

      peer.registerMethod('subscribe', (json_rpc_2.Parameters params) async {
        var requestId = params['request_id'].asString;
        var clientId = params['client_id'].asString;
        var eventName = params['event_name'].asString;
        var rq = new _JsonRpc2SubscriptionRequestImpl(
            clientId, eventName, requestId, peer, _uuid);
        _onSubscribe.add(rq);
      });

      peer.registerMethod('unsubscribe', (json_rpc_2.Parameters params) async {
        var requestId = params['request_id'].asString;
        var clientId = params['client_id'].asString;
        var subscriptionId = params['subscription_id'].asString;
        var rq = new _JsonRpc2UnsubscriptionRequestImpl(
            clientId, subscriptionId, peer, requestId);
        _onUnsubscribe.add(rq);
      });

      peer.listen();
    });
  }
}

class _JsonRpc2PublishRequestImpl extends PublishRequest {
  final String requestId, clientId, eventName;
  final value;
  final json_rpc_2.Peer peer;

  _JsonRpc2PublishRequestImpl(
      this.requestId, this.clientId, this.eventName, this.value, this.peer);

  @override
  void accept(PublishResponse response) {
    peer.sendNotification(requestId, {
      'status': true,
      'request_id': requestId,
      'result': {'listeners': response.listeners}
    });
  }

  @override
  void reject(String errorMessage) {
    peer.sendNotification(requestId, {
      'status': false,
      'request_id': requestId,
      'error_message': errorMessage
    });
  }
}

class _JsonRpc2SubscriptionRequestImpl extends SubscriptionRequest {
  @override
  final String clientId, eventName;

  final String requestId;

  final json_rpc_2.Peer peer;

  final Uuid _uuid;

  _JsonRpc2SubscriptionRequestImpl(
      this.clientId, this.eventName, this.requestId, this.peer, this._uuid);

  @override
  FutureOr<Subscription> accept() {
    var id = _uuid.v4() as String;
    peer.sendNotification(requestId,
        {'status': true, 'request_id': requestId, 'subscription_id': id});
    return new _JsonRpc2SubscriptionImpl(clientId, id, eventName, peer);
  }

  @override
  void reject(String errorMessage) {
    peer.sendNotification(requestId, {
      'status': false,
      'request_id': requestId,
      'error_message': errorMessage
    });
  }
}

class _JsonRpc2SubscriptionImpl extends Subscription {
  @override
  final String clientId, id;

  final String eventName;

  final json_rpc_2.Peer peer;

  _JsonRpc2SubscriptionImpl(this.clientId, this.id, this.eventName, this.peer);

  @override
  void dispatch(event) {
    peer.sendNotification('event', {'event_name': eventName, 'value': event});
  }
}

class _JsonRpc2UnsubscriptionRequestImpl extends UnsubscriptionRequest {
  @override
  final String clientId;

  @override
  final String subscriptionId;

  final json_rpc_2.Peer peer;

  final String requestId;

  _JsonRpc2UnsubscriptionRequestImpl(
      this.clientId, this.subscriptionId, this.peer, this.requestId);

  @override
  accept() {
    peer.sendNotification(requestId, {'status': true, 'result': {}});
  }

  @override
  void reject(String errorMessage) {
    peer.sendNotification(requestId, {
      'status': false,
      'request_id': requestId,
      'error_message': errorMessage
    });
  }
}
